/******************************************************************************\
**
**     COMPANY:	LDW Software
**
**      MODULE:	Construction 
**
** DESCRIPTION:	Construction of houses
**
**
\******************************************************************************/

#ifndef CONSTRUCTION_H
#define CONSTRUCTION_H

/******************************************************************************\
** LIBRARY INCLUDES
\******************************************************************************/
#include "ldwImage.h"
#include "Puzzle.h"
#include "../Utils/theGraphicsManager.h"
#include "../World/ContentMap.h"

/******************************************************************************\
** CONSTANT DEFINITIONS
\******************************************************************************/
const int Construction_c_iMildDisrepair        = 250;
const int Construction_c_iSignificantDisrepair = 1600; //was 600
const int Construction_c_iSevereDisrepair      = 2000;	//was 1000

/******************************************************************************\
** CLASS ENUMERATIONS
\******************************************************************************/
enum EConstruction
{
	eConstruction_House1 = 0,
	eConstruction_House2,
	eConstruction_House3,
	eConstruction_ClothingHut,
	eConstruction_LoveShack,
	eConstruction_Nursery,
#ifdef PUZZLE_ENABLE_HOSPITAL
	eConstruction_Hospital,
#endif
	eConstruction_NumberOf
};

enum EDisrepair
{
	eDisrepair_None = 0,
	eDisrepair_Mild,
	eDisrepair_Significant,
	eDisrepair_Severe,
	eDisrepair_NotBuilt,

	eDisrepair_NumberOf
};

class CHouse;

/******************************************************************************\
** CLASS DEFINITIONS
\******************************************************************************/
class CConstruction
{
public:

	//class methods
	//setup
	CConstruction(void);

	const EPuzzle GetPuzzle(const EConstruction eConstruction);
	CHouse &GetHouse(EConstruction eConstruction) 						{return *m_houseList[eConstruction];}

	bool CanMove(CContentMap::EObject houseObject);
	void PickupHut(CContentMap::EObject houseObject);
	void MoveHut(ldwPoint offset);
	void DropHut();

	void MouseOnHut(CContentMap::EObject houseObject);

	void InflictEarthQuakeDamage(const EConstruction eConstruction);

private:

	static CHouse* m_houseList[eConstruction_NumberOf];

};

class CHouse : public IPuzzle
{
	friend class CPuzzle;

public:

	//class methods
	//setup

	// from IPuzzle

	void 	  Reset();
	void 	  LoadAssets();
	void 	  ReleaseAssets();
	const int LoadState(const unsigned char *pData);
	const int SaveState(const unsigned char *pData, const int iMaxDataSize) const;

	const int SaveStateSize() {return sizeof (SSaveState);}

	// process
	void Refresh();
	void Draw() const;
	void Draw(const int iRef);
	void Update();
	void Advance(CVillager &Villager, const int iRef);

	int  InitialPoints();
	const int ProgressMultiplier(void) const;

	// end of IPuzzle methods

	// access
	virtual const EPuzzle    GetPuzzle() const {return m_puzzle;}
	virtual const EDisrepair StateOfDisrepair() const;
	virtual void             SetDisrepair(const EDisrepair eDisrepair);

	virtual bool CanMove();
	virtual void PickupHut();
	virtual void MoveHut(ldwPoint offset);
	virtual void DropHut();

	virtual ldwPoint GetPos() {return m_sConstruction.m_location;}

	virtual void MouseOnHut(CContentMap::EObject houseObject);
	
	virtual CContentMap::EObject SiteObject()   {return m_mapSiteObject;}
	virtual CContentMap::EObject Object() 		{return m_mapObject;}

protected:

	CHouse();

	virtual bool NeedsPlacement() {return false;}

	virtual void PlaceHut();

	virtual bool IsLegalHutSpot(ldwPoint location);

	virtual void UpdateContentMap();

	virtual const bool IsComplete() const;
	virtual const int PercentComplete() const;

	//class structures
	struct SConstruction
	{
		ldwPoint		m_location;
		bool			m_bIsInProgress;
		EDisrepair		m_eInitialDisrepair;
		int				m_iInitialPointsRemaining;
		int				m_iSmoke;
		int				m_iFrame;
	};

	struct SSaveState
	{
		SConstruction	m_sConstruction;
	};

	EConstruction		  m_construction;
	EPuzzle			      m_puzzle;
	EImage				  m_houseImageId;
	CContentMap::EObject  m_mapObject;
	CContentMap::EObject  m_mapSiteObject;
	CContentMap::EObject  m_mapDoorObject;
	CContentMap::EHotSpot m_mapHotSpot;

	int				      m_sceneOffsetY;

	int	m_progressMutliplier;

	//class data
	SConstruction	m_sConstruction;
	
	ldwPoint		m_chimneyOffset;
	ldwPoint		m_doorOffset;

	bool			m_hutOnTheMove;
	bool			m_hutHasMouse;

	ldwPoint		m_preMoveLocation;
	bool			m_canDropHouse;
	bool			m_bNeedsContentMapUpdate;

	ldwImage *		m_pScaffolding;
	ldwImageGrid *	m_pHouse;

	char *          m_pHouseMapMask;
	char *			m_pHotSpotMask;
	int				m_houseMaskWidth;
	int             m_houseMaskHeight;
	int				m_houseMaskCell;
	int				m_hotSpotMaskCell;

	float			m_alphaAngle;
};

class CHouse1 : public CHouse
{
public:

	CHouse1();

	void	Reset();

protected:

	bool NeedsPlacement() {return StateOfDisrepair() == eDisrepair_NotBuilt;}
};

class CHouse2 : public CHouse
{
public:

	CHouse2();

	void	Reset();

protected:

	bool NeedsPlacement();
};

class CHouse3 : public CHouse
{
public:

	CHouse3();

	void	Reset();

protected:

	bool NeedsPlacement();
};

class CClothingHut : public CHouse
{
public:

	CClothingHut();

	void	Reset();
	void 	Draw() const;
	void 	Draw(const int iRef);

protected:

	bool NeedsPlacement();
};

class CLoveShack : public CHouse
{
public:

	CLoveShack();

	void	Reset();

protected:

	bool NeedsPlacement() {return StateOfDisrepair() == eDisrepair_NotBuilt;}
};

class CNurseryHut : public CHouse
{
public:

	CNurseryHut();

	void	Reset();

protected:

	bool NeedsPlacement();
};

#ifdef PUZZLE_ENABLE_HOSPITAL

class CHospitalHut : public CHouse
{
public:

	CHospitalHut();

	void	Reset();

	virtual void PickupHut();
	virtual void UpdateContentMap();

protected:

	bool NeedsPlacement();

	void UpdateObjectSpots(bool addObjects);
};

#endif

/******************************************************************************\
** GLOBAL DECLARATIONS
\******************************************************************************/

extern CHouse1				House1;
extern CHouse2				House2;
extern CHouse3				House3;
extern CClothingHut			ClothingHut;
extern CLoveShack			LoveShack;
extern CNurseryHut			NurseryHut;

#ifdef PUZZLE_ENABLE_HOSPITAL
extern CHospitalHut			HospitalHut;
#endif

extern CConstruction		Construction;

#endif

/******************************************************************************\
** END OF FILE
\******************************************************************************/